"""
Spotify MCP Server 
Provides Spotify integration through Model Context Protocol 
"""

import os
import logging
from typing import List, Dict, Any, Optional

import spotipy
from spotipy.oauth2 import SpotifyOAuth
from mcp.server.fastmcp.server import FastMCP
from pydantic import BaseModel
from dotenv import load_dotenv

from mcp_tools import search_artists_tool, get_artist_top_tracks_tool, get_artist_tool, get_artist_albums_tool, ArtistSearchRequest
from mcp_tools import create_playlist_tool, add_tracks_to_playlist_tool, get_user_playlists_tool, PlaylistCreateRequest, AddTracksRequest
from mcp_tools import get_album_tool, get_album_tracks_tool, get_new_releases_tool
from mcp_tools import get_user_top_artists_tool, get_user_top_tracks_tool

load_dotenv()

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

class SimilarArtistPlaylistRequest(BaseModel):
    artist_name: str
    playlist_name: str
    playlist_description: Optional[str] = ""
    track_limit: int = 20
    include_artist_tracks: bool = True

app = FastMCP("Spotify MCP Server")

spotify = None

def _setup_spotify_client():
    """Initialize Spotify client with OAuth"""
    global spotify
    try:
        client_id = os.getenv('SPOTIFY_CLIENT_ID')
        client_secret = os.getenv('SPOTIFY_CLIENT_SECRET')
        redirect_uri = os.getenv('SPOTIFY_REDIRECT_URI', 'http://localhost:8888/callback')
        
        if not client_id or not client_secret:
            raise ValueError("Spotify credentials not found in environment variables")
        
        # Setup Spotify OAuth
        scope = "playlist-modify-public playlist-modify-private user-library-read user-read-private user-top-read"
        
        auth_manager = SpotifyOAuth(
            client_id=client_id,
            client_secret=client_secret,
            redirect_uri=redirect_uri,
            scope=scope,
            cache_path=".spotify_cache"
        )
        
        spotify = spotipy.Spotify(auth_manager=auth_manager)
        logger.info("Spotify client initialized successfully")
        
    except Exception as e:
        logger.error(f"Failed to initialize Spotify client: {e}")
        raise

_setup_spotify_client()

# Create wrapper functions that preserve names and documentation
async def create_playlist(request: PlaylistCreateRequest) -> Dict[str, Any]:
    return await create_playlist_tool(spotify, request)

async def search_artists(request: ArtistSearchRequest) -> Dict[str, Any]:
    return await search_artists_tool(spotify, request)

async def get_artist(artist_id: str) -> Dict[str, Any]:
    return await get_artist_tool(spotify, artist_id)

async def get_artist_albums(artist_id: str, include_groups: str = "album,single", limit: int = 20) -> Dict[str, Any]:
    return await get_artist_albums_tool(spotify, artist_id, include_groups, limit)

async def get_artist_top_tracks(artist_id: str, market: str = "US") -> Dict[str, Any]:
    return await get_artist_top_tracks_tool(spotify, artist_id, market)

async def add_tracks_to_playlist(request: AddTracksRequest) -> Dict[str, Any]:
    return await add_tracks_to_playlist_tool(spotify, request)

async def get_user_playlists() -> Dict[str, Any]:
    return await get_user_playlists_tool(spotify)

async def get_album(album_id: str, market: str = "US") -> Dict[str, Any]:
    return await get_album_tool(spotify, album_id, market)

async def get_album_tracks(album_id: str, market: str = "US", limit: int = 20, offset: int = 0) -> Dict[str, Any]:
    return await get_album_tracks_tool(spotify, album_id, market, limit, offset)

async def get_new_releases(country: str = "US", limit: int = 20, offset: int = 0) -> Dict[str, Any]:
    return await get_new_releases_tool(spotify, country, limit, offset)

async def get_user_top_artists(time_range: str = "medium_term", limit: int = 20, offset: int = 0) -> Dict[str, Any]:
    return await get_user_top_artists_tool(spotify, time_range, limit, offset)

async def get_user_top_tracks(time_range: str = "medium_term", limit: int = 20, offset: int = 0) -> Dict[str, Any]:
    return await get_user_top_tracks_tool(spotify, time_range, limit, offset)

# Copy docstrings from original functions
create_playlist.__doc__ = create_playlist_tool.__doc__
search_artists.__doc__ = search_artists_tool.__doc__
get_artist.__doc__ = get_artist_tool.__doc__
get_artist_albums.__doc__ = get_artist_albums_tool.__doc__
get_artist_top_tracks.__doc__ = get_artist_top_tracks_tool.__doc__
add_tracks_to_playlist.__doc__ = add_tracks_to_playlist_tool.__doc__
get_user_playlists.__doc__ = get_user_playlists_tool.__doc__
get_album.__doc__ = get_album_tool.__doc__
get_album_tracks.__doc__ = get_album_tracks_tool.__doc__
get_new_releases.__doc__ = get_new_releases_tool.__doc__
get_user_top_artists.__doc__ = get_user_top_artists_tool.__doc__
get_user_top_tracks.__doc__ = get_user_top_tracks_tool.__doc__

# Register tools with FastMCP
app.tool()(create_playlist)
app.tool()(search_artists)
app.tool()(get_artist)
app.tool()(get_artist_albums)
app.tool()(get_artist_top_tracks)
app.tool()(add_tracks_to_playlist)
app.tool()(get_user_playlists)
app.tool()(get_album)
app.tool()(get_album_tracks)
app.tool()(get_new_releases)
app.tool()(get_user_top_artists)
app.tool()(get_user_top_tracks)

def main():
    """Entry point for the MCP server"""
    app.run()

if __name__ == "__main__":
    main()



"""
MCP Tools package for Spotify integration
"""

from .artist_tools import search_artists_tool, get_artist_top_tracks_tool, get_artist_tool, get_artist_albums_tool, ArtistSearchRequest
from .playlist_tools import create_playlist_tool, add_tracks_to_playlist_tool, get_user_playlists_tool, PlaylistCreateRequest, AddTracksRequest
from .albums import get_album_tool, get_album_tracks_tool, get_new_releases_tool
from .user_tools import get_user_top_artists_tool, get_user_top_tracks_tool

__all__ = [
    'search_artists_tool',
    'get_artist_top_tracks_tool',
    'get_artist_tool',
    'get_artist_albums_tool',
    'ArtistSearchRequest',
    'create_playlist_tool',
    'add_tracks_to_playlist_tool',
    'get_user_playlists_tool',
    'PlaylistCreateRequest',
    'AddTracksRequest',
    'get_album_tool',
    'get_album_tracks_tool',
    'get_new_releases_tool',
    'get_user_top_artists_tool',
    'get_user_top_tracks_tool'
] 


"""
Artist-related tools for Spotify MCP Server
"""

import logging
from typing import Dict, Any
from pydantic import BaseModel

logger = logging.getLogger(__name__)

class ArtistSearchRequest(BaseModel):
    query: str
    limit: int = 10

async def search_artists_tool(spotify_client, request: ArtistSearchRequest) -> Dict[str, Any]:
    """Search for artists on Spotify
    
    Arguments:
        request (ArtistSearchRequest):
            - query (str): Search query for artist name
            - limit (int, optional): Maximum number of results to return (default: 10, max: 50)
    
    Returns:
        Dict[str, Any]:
            - success (bool): Whether the operation was successful
            - artists (List[Dict]): List of artist objects containing:
                - id (str): Spotify artist ID
                - name (str): Artist name
                - popularity (int): Artist popularity score (0-100)
                - followers (int): Number of followers
                - genres (List[str]): List of genres
                - spotify_url (str): Spotify profile URL
            - total_found (int): Number of artists found
            - message (str): Success message
            - error (str, optional): Error message if failed
    """
    try:
        results = spotify_client.search(
            q=request.query,
            type='artist',
            limit=min(request.limit, 50)
        )
        
        artists = []
        for artist in results['artists']['items']:
            artists.append({
                "id": artist['id'],
                "name": artist['name'],
                "popularity": artist['popularity'],
                "followers": artist['followers']['total'],
                "genres": artist['genres'],
                "spotify_url": artist['external_urls']['spotify']
            })
        
        return {
            "success": True,
            "artists": artists,
            "total_found": len(artists),
            "message": f"Found {len(artists)} artists for '{request.query}'"
        }
        
    except Exception as e:
        logger.error(f"Error searching artists: {e}")
        return {
            "success": False,
            "error": str(e),
            "message": "Failed to search artists"
        }

async def get_artist_tool(spotify_client, artist_id: str) -> Dict[str, Any]:
    """Get detailed information about an artist
    
    Arguments:
        artist_id (str): Spotify artist ID to get information for
    
    Returns:
        Dict[str, Any]:
            - success (bool): Whether the operation was successful
            - artist (Dict): Detailed artist information containing:
                - id (str): Spotify artist ID
                - name (str): Artist name
                - popularity (int): Artist popularity score (0-100)
                - followers (int): Number of followers
                - genres (List[str]): List of genres
                - spotify_url (str): Spotify profile URL
                - images (List[Dict]): List of artist images with:
                    - url (str): Image URL
                    - height (int): Image height
                    - width (int): Image width
            - message (str): Success message
            - error (str, optional): Error message if failed
    """
    try:
        artist = spotify_client.artist(artist_id)
        
        return {
            "success": True,
            "artist": {
                "id": artist['id'],
                "name": artist['name'],
                "popularity": artist['popularity'],
                "followers": artist['followers']['total'],
                "genres": artist['genres'],
                "spotify_url": artist['external_urls']['spotify'],
                "images": artist.get('images', [])
            },
            "message": f"Retrieved information for {artist['name']}"
        }
        
    except Exception as e:
        logger.error(f"Error getting artist: {e}")
        return {
            "success": False,
            "error": str(e),
            "message": "Failed to get artist information"
        }

async def get_artist_albums_tool(spotify_client, artist_id: str, include_groups: str = "album,single", limit: int = 20) -> Dict[str, Any]:
    """Get albums for an artist
    
    Arguments:
        artist_id (str): Spotify artist ID to get albums for
        include_groups (str, optional): Album types to include (default: "album,single")
            - "album": Full albums
            - "single": Singles
            - "appears_on": Appearances on other albums
            - "compilation": Compilation albums
        limit (int, optional): Maximum number of albums to return (default: 20, max: 50)
    
    Returns:
        Dict[str, Any]:
            - success (bool): Whether the operation was successful
            - artist (Dict): Basic artist information:
                - id (str): Artist ID
                - name (str): Artist name
            - albums (List[Dict]): List of album objects containing:
                - id (str): Spotify album ID
                - name (str): Album name
                - album_type (str): Type of album (album, single, compilation)
                - release_date (str): Release date
                - total_tracks (int): Number of tracks
                - spotify_url (str): Spotify album URL
                - images (List[Dict]): Album cover images
            - total_albums (int): Number of albums found
            - message (str): Success message
            - error (str, optional): Error message if failed
    """
    try:
        artist = spotify_client.artist(artist_id)
        albums = spotify_client.artist_albums(
            artist_id,
            album_type=include_groups,
            limit=min(limit, 50)
        )
        
        album_list = []
        for album in albums['items']:
            album_list.append({
                "id": album['id'],
                "name": album['name'],
                "album_type": album['album_type'],
                "release_date": album['release_date'],
                "total_tracks": album['total_tracks'],
                "spotify_url": album['external_urls']['spotify'],
                "images": album.get('images', [])
            })
        
        return {
            "success": True,
            "artist": {
                "id": artist['id'],
                "name": artist['name']
            },
            "albums": album_list,
            "total_albums": len(album_list),
            "message": f"Found {len(album_list)} albums for {artist['name']}"
        }
        
    except Exception as e:
        logger.error(f"Error getting artist albums: {e}")
        return {
            "success": False,
            "error": str(e),
            "message": "Failed to get artist albums"
        }

async def get_artist_top_tracks_tool(spotify_client, artist_id: str, market: str = "US") -> Dict[str, Any]:
    """Get top tracks for an artist
    
    Arguments:
        artist_id (str): Spotify artist ID to get top tracks for
        market (str, optional): Market/country code for track availability (default: "US")
    
    Returns:
        Dict[str, Any]:
            - success (bool): Whether the operation was successful
            - artist (Dict): Information about the artist:
                - id (str): Artist ID
                - name (str): Artist name
            - tracks (List[Dict]): List of track objects containing:
                - id (str): Spotify track ID
                - name (str): Track name
                - album (str): Album name
                - popularity (int): Track popularity score (0-100)
                - duration_ms (int): Track duration in milliseconds
                - spotify_url (str): Spotify track URL
            - message (str): Success message
            - error (str, optional): Error message if failed
    """
    try:
        artist = spotify_client.artist(artist_id)
        tracks = spotify_client.artist_top_tracks(artist_id, country=market)
        
        track_list = []
        for track in tracks['tracks']:
            track_list.append({
                "id": track['id'],
                "name": track['name'],
                "album": track['album']['name'],
                "popularity": track['popularity'],
                "duration_ms": track['duration_ms'],
                "spotify_url": track['external_urls']['spotify']
            })
        
        return {
            "success": True,
            "artist": {
                "id": artist['id'],
                "name": artist['name']
            },
            "tracks": track_list,
            "message": f"Found {len(track_list)} top tracks for {artist['name']}"
        }
        
    except Exception as e:
        logger.error(f"Error getting artist top tracks: {e}")
        return {
            "success": False,
            "error": str(e),
            "message": "Failed to get artist top tracks"
        } 


"""
Album-related tools for Spotify MCP Server
"""

import logging
from typing import Dict, Any, List, Optional
from pydantic import BaseModel

logger = logging.getLogger(__name__)

async def get_album_tool(spotify_client, album_id: str, market: str = "US") -> Dict[str, Any]:
    """Get detailed information about an album
    
    Arguments:
        album_id (str): Spotify album ID to get information for
        market (str, optional): Market/country code for album availability (default: "US")
    
    Returns:
        Dict[str, Any]:
            - success (bool): Whether the operation was successful
            - album (Dict): Detailed album information containing:
                - id (str): Spotify album ID
                - name (str): Album name
                - album_type (str): Type of album (album, single, compilation)
                - release_date (str): Release date
                - total_tracks (int): Number of tracks
                - popularity (int): Album popularity score (0-100)
                - spotify_url (str): Spotify album URL
                - artists (List[Dict]): List of artists with:
                    - id (str): Artist ID
                    - name (str): Artist name
                - images (List[Dict]): Album cover images
                - genres (List[str]): List of genres
            - message (str): Success message
            - error (str, optional): Error message if failed
    """
    try:
        album = spotify_client.album(album_id, market=market)
        
        artists = []
        for artist in album.get('artists', []):
            artists.append({
                "id": artist['id'],
                "name": artist['name']
            })
        
        return {
            "success": True,
            "album": {
                "id": album['id'],
                "name": album['name'],
                "album_type": album['album_type'],
                "release_date": album['release_date'],
                "total_tracks": album['total_tracks'],
                "popularity": album.get('popularity', 0),
                "spotify_url": album['external_urls']['spotify'],
                "artists": artists,
                "images": album.get('images', []),
                "genres": album.get('genres', [])
            },
            "message": f"Retrieved information for album '{album['name']}'"
        }
        
    except Exception as e:
        logger.error(f"Error getting album: {e}")
        return {
            "success": False,
            "error": str(e),
            "message": "Failed to get album information"
        }

async def get_album_tracks_tool(spotify_client, album_id: str, market: str = "US", limit: int = 20, offset: int = 0) -> Dict[str, Any]:
    """Get tracks from an album
    
    Arguments:
        album_id (str): Spotify album ID to get tracks for
        market (str, optional): Market/country code for track availability (default: "US")
        limit (int, optional): Maximum number of tracks to return (default: 20, max: 50)
        offset (int, optional): Index of the first track to return (default: 0)
    
    Returns:
        Dict[str, Any]:
            - success (bool): Whether the operation was successful
            - album (Dict): Basic album information:
                - id (str): Album ID
                - name (str): Album name
            - tracks (List[Dict]): List of track objects containing:
                - id (str): Spotify track ID
                - name (str): Track name
                - track_number (int): Track number on the album
                - duration_ms (int): Track duration in milliseconds
                - spotify_url (str): Spotify track URL
                - artists (List[Dict]): List of artists with:
                    - id (str): Artist ID
                    - name (str): Artist name
            - total_tracks (int): Total number of tracks in the album
            - message (str): Success message
            - error (str, optional): Error message if failed
    """
    try:
        album = spotify_client.album(album_id, market=market)
        tracks = spotify_client.album_tracks(
            album_id,
            market=market,
            limit=min(limit, 50),
            offset=offset
        )
        
        track_list = []
        for track in tracks['items']:
            track_artists = []
            for artist in track.get('artists', []):
                track_artists.append({
                    "id": artist['id'],
                    "name": artist['name']
                })
            
            track_list.append({
                "id": track['id'],
                "name": track['name'],
                "track_number": track['track_number'],
                "duration_ms": track['duration_ms'],
                "spotify_url": track['external_urls']['spotify'],
                "artists": track_artists
            })
        
        return {
            "success": True,
            "album": {
                "id": album['id'],
                "name": album['name']
            },
            "tracks": track_list,
            "total_tracks": tracks['total'],
            "message": f"Found {len(track_list)} tracks from album '{album['name']}'"
        }
        
    except Exception as e:
        logger.error(f"Error getting album tracks: {e}")
        return {
            "success": False,
            "error": str(e),
            "message": "Failed to get album tracks"
        }

async def get_new_releases_tool(spotify_client, country: str = "US", limit: int = 20, offset: int = 0) -> Dict[str, Any]:
    """Get new album releases
    
    Arguments:
        country (str, optional): Country code to get new releases for (default: "US")
        limit (int, optional): Maximum number of albums to return (default: 20, max: 50)
        offset (int, optional): Index of the first album to return (default: 0)
    
    Returns:
        Dict[str, Any]:
            - success (bool): Whether the operation was successful
            - albums (List[Dict]): List of new release albums containing:
                - id (str): Spotify album ID
                - name (str): Album name
                - album_type (str): Type of album (album, single, compilation)
                - release_date (str): Release date
                - total_tracks (int): Number of tracks
                - spotify_url (str): Spotify album URL
                - artists (List[Dict]): List of artists with:
                    - id (str): Artist ID
                    - name (str): Artist name
                - images (List[Dict]): Album cover images
            - total_albums (int): Number of albums found
            - message (str): Success message
            - error (str, optional): Error message if failed
    """
    try:
        new_releases = spotify_client.new_releases(
            country=country,
            limit=min(limit, 50),
            offset=offset
        )
        
        album_list = []
        for album in new_releases['albums']['items']:
            artists = []
            for artist in album.get('artists', []):
                artists.append({
                    "id": artist['id'],
                    "name": artist['name']
                })
            
            album_list.append({
                "id": album['id'],
                "name": album['name'],
                "album_type": album['album_type'],
                "release_date": album['release_date'],
                "total_tracks": album['total_tracks'],
                "spotify_url": album['external_urls']['spotify'],
                "artists": artists,
                "images": album.get('images', [])
            })
        
        return {
            "success": True,
            "albums": album_list,
            "total_albums": len(album_list),
            "message": f"Found {len(album_list)} new releases for {country}"
        }
        
    except Exception as e:
        logger.error(f"Error getting new releases: {e}")
        return {
            "success": False,
            "error": str(e),
            "message": "Failed to get new releases"
        } 


"""
Playlist-related tools for Spotify MCP Server
"""

import logging
from typing import Dict, Any, List, Optional
from pydantic import BaseModel

logger = logging.getLogger(__name__)

class PlaylistCreateRequest(BaseModel):
    name: str
    description: Optional[str] = ""
    public: bool = False
    collaborative: bool = False

class AddTracksRequest(BaseModel):
    playlist_id: str
    track_uris: List[str]

async def create_playlist_tool(spotify_client, request: PlaylistCreateRequest) -> Dict[str, Any]:
    """Create a new Spotify playlist
    
    Arguments:
        request (PlaylistCreateRequest): 
            - name (str): Name of the playlist to create
            - description (str, optional): Description of the playlist (default: "")
            - public (bool, optional): Whether the playlist is public (default: False)
            - collaborative (bool, optional): Whether the playlist is collaborative (default: False)
    
    Returns:
        Dict[str, Any]: 
            - success (bool): Whether the operation was successful
            - playlist_id (str): ID of the created playlist
            - playlist_name (str): Name of the created playlist
            - playlist_url (str): Spotify URL of the playlist
            - message (str): Success message
            - error (str, optional): Error message if failed
    """
    try:
        user = spotify_client.current_user()
        playlist = spotify_client.user_playlist_create(
            user=user['id'],
            name=request.name,
            public=request.public,
            collaborative=request.collaborative,
            description=request.description
        )
        
        return {
            "success": True,
            "playlist_id": playlist['id'],
            "playlist_name": playlist['name'],
            "playlist_url": playlist['external_urls']['spotify'],
            "message": f"Playlist '{request.name}' created successfully"
        }
        
    except Exception as e:
        logger.error(f"Error creating playlist: {e}")
        return {
            "success": False,
            "error": str(e),
            "message": "Failed to create playlist"
        }

async def add_tracks_to_playlist_tool(spotify_client, request: AddTracksRequest) -> Dict[str, Any]:
    """Add tracks to an existing playlist
    
    Arguments:
        request (AddTracksRequest):
            - playlist_id (str): Spotify playlist ID to add tracks to
            - track_uris (List[str]): List of Spotify track URIs to add
    
    Returns:
        Dict[str, Any]:
            - success (bool): Whether the operation was successful
            - playlist_id (str): ID of the playlist
            - tracks_added (int): Number of tracks added
            - message (str): Success message
            - error (str, optional): Error message if failed
    """
    try:
        spotify_client.playlist_add_items(request.playlist_id, request.track_uris)
        
        return {
            "success": True,
            "playlist_id": request.playlist_id,
            "tracks_added": len(request.track_uris),
            "message": f"Added {len(request.track_uris)} tracks to playlist"
        }
        
    except Exception as e:
        logger.error(f"Error adding tracks to playlist: {e}")
        return {
            "success": False,
            "error": str(e),
            "message": "Failed to add tracks to playlist"
        }

async def get_user_playlists_tool(spotify_client) -> Dict[str, Any]:
    """Get current user's playlists
    
    Arguments:
        None (no parameters required)
    
    Returns:
        Dict[str, Any]:
            - success (bool): Whether the operation was successful
            - playlists (List[Dict]): List of playlist objects containing:
                - id (str): Spotify playlist ID
                - name (str): Playlist name
                - description (str): Playlist description
                - public (bool): Whether the playlist is public
                - tracks_count (int): Number of tracks in the playlist
                - spotify_url (str): Spotify playlist URL
            - total_playlists (int): Total number of playlists
            - message (str): Success message
            - error (str, optional): Error message if failed
    """
    try:
        user = spotify_client.current_user()
        playlists = spotify_client.current_user_playlists()
        
        playlist_list = []
        for playlist in playlists['items']:
            playlist_list.append({
                "id": playlist['id'],
                "name": playlist['name'],
                "description": playlist.get('description', ''),
                "public": playlist['public'],
                "tracks_count": playlist['tracks']['total'],
                "spotify_url": playlist['external_urls']['spotify']
            })
        
        return {
            "success": True,
            "playlists": playlist_list,
            "total_playlists": len(playlist_list),
            "message": f"Found {len(playlist_list)} playlists for user {user['display_name']}"
        }
        
    except Exception as e:
        logger.error(f"Error getting user playlists: {e}")
        return {
            "success": False,
            "error": str(e),
            "message": "Failed to get user playlists"
        } 


"""
User-related tools for Spotify MCP Server
"""

import logging
from typing import Dict, Any

logger = logging.getLogger(__name__)
valid_time_ranges = ["short_term", "medium_term", "long_term"]

async def get_user_top_artists_tool(
    spotify_client,
    time_range: str = "medium_term",
    limit: int = 20,
    offset: int = 0
) -> Dict[str, Any]:
    """Get user's top artists from Spotify
    
    Arguments:
        time_range (str, optional): Time period for top artists (default: "medium_term")
            - "short_term": Last 4 weeks
            - "medium_term": Last 6 months  
            - "long_term": Several years
        limit (int, optional): Maximum number of artists to return (default: 20, max: 50)
        offset (int, optional): Index of the first artist to return (default: 0)
    
    Returns:
        Dict[str, Any]:
            - success (bool): Whether the operation was successful
            - artists (List[Dict]): List of artist objects containing:
                - id (str): Spotify artist ID
                - name (str): Artist name
                - popularity (int): Artist popularity score (0-100)
                - followers (int): Number of followers
                - genres (List[str]): List of genres
                - spotify_url (str): Spotify profile URL
                - images (List[Dict]): Artist images with url, height, width
            - time_range (str): Time range used for the query
            - total_artists (int): Number of artists returned
            - message (str): Success message
            - error (str, optional): Error message if failed
    """
    try:
        if time_range not in valid_time_ranges:
            return {
                "success": False,
                "error": f"Invalid time_range '{time_range}'. Valid options are: {', '.join(valid_time_ranges)}",
                "message": "Invalid time range parameter"
            }
        
        if offset < 0:
            return {
                "success": False,
                "error": "Offset must be non-negative",
                "message": "Invalid offset parameter"
            }
        
        if limit > 50:
            limit = 50
        
        if limit < 1:
            return {
                "success": False,
                "error": "Limit must be at least 1",
                "message": "Invalid limit parameter"
            }
        
        results = spotify_client.current_user_top_artists(
            time_range=time_range,
            limit=limit,
            offset=offset
        )
        
        artists = []
        for artist in results['items']:
            artists.append({
                "id": artist['id'],
                "name": artist['name'],
                "popularity": artist['popularity'],
                "followers": artist['followers']['total'],
                "genres": artist['genres'],
                "spotify_url": artist['external_urls']['spotify'],
                "images": artist.get('images', [])
            })
        
        return {
            "success": True,
            "artists": artists,
            "time_range": time_range,
            "total_artists": len(artists),
            "message": f"Found {len(artists)} top artists for {time_range} period"
        }
        
    except Exception as e:
        logger.error(f"Error getting user top artists: {e}")
        return {
            "success": False,
            "error": str(e),
            "message": "Failed to get user's top artists"
        }

async def get_user_top_tracks_tool(
    spotify_client,
    time_range: str = "medium_term",
    limit: int = 20,
    offset: int = 0
) -> Dict[str, Any]:
    """Get user's top tracks from Spotify
    
    Arguments:
        time_range (str, optional): Time period for top tracks (default: "medium_term")
            - "short_term": Last 4 weeks
            - "medium_term": Last 6 months
            - "long_term": Several years
        limit (int, optional): Maximum number of tracks to return (default: 20, max: 50)
        offset (int, optional): Index of the first track to return (default: 0)
    
    Returns:
        Dict[str, Any]:
            - success (bool): Whether the operation was successful
            - tracks (List[Dict]): List of track objects containing:
                - id (str): Spotify track ID
                - name (str): Track name
                - album (Dict): Album info with id, name, images
                - artists (List[Dict]): List of artists with id, name
                - popularity (int): Track popularity score (0-100)
                - duration_ms (int): Track duration in milliseconds
                - spotify_url (str): Spotify track URL
            - time_range (str): Time range used for the query
            - total_tracks (int): Number of tracks returned
            - message (str): Success message
            - error (str, optional): Error message if failed
    """
    try:
        if time_range not in valid_time_ranges:
            return {
                "success": False,
                "error": f"Invalid time_range '{time_range}'. Valid options are: {', '.join(valid_time_ranges)}",
                "message": "Invalid time range parameter"
            }
        
        if offset < 0:
            return {
                "success": False,
                "error": "Offset must be non-negative",
                "message": "Invalid offset parameter"
            }
        
        if limit > 50:
            limit = 50
        
        if limit < 1:
            return {
                "success": False,
                "error": "Limit must be at least 1",
                "message": "Invalid limit parameter"
            }
        
        results = spotify_client.current_user_top_tracks(
            time_range=time_range,
            limit=limit,
            offset=offset
        )
        
        tracks = []
        for track in results['items']:
            album_info = {
                "id": track['album']['id'],
                "name": track['album']['name'],
                "images": track['album'].get('images', [])
            }
            
            track_artists = []
            for artist in track.get('artists', []):
                track_artists.append({
                    "id": artist['id'],
                    "name": artist['name']
                })
            
            tracks.append({
                "id": track['id'],
                "name": track['name'],
                "album": album_info,
                "artists": track_artists,
                "popularity": track['popularity'],
                "duration_ms": track['duration_ms'],
                "spotify_url": track['external_urls']['spotify']
            })
        
        return {
            "success": True,
            "tracks": tracks,
            "time_range": time_range,
            "total_tracks": len(tracks),
            "message": f"Found {len(tracks)} top tracks for {time_range} period"
        }
        
    except Exception as e:
        logger.error(f"Error getting user top tracks: {e}")
        return {
            "success": False,
            "error": str(e),
            "message": "Failed to get user's top tracks"
        }